package com.compomics.marc;

import com.compomics.util.experiment.biology.AminoAcidPattern;
import com.compomics.util.preferences.SequenceMatchingPreferences;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

/**
 * Maps the tag results on search engine results
 *
 * @author Marc
 */
public class TagMapping {

    private static final String separator = "\t";

    private HashMap<String, String> searchEngineTags = new HashMap<String, String>();
    private HashMap<String, String> searchEnginePeptides = new HashMap<String, String>();
    private HashMap<String, String> searchEngineModifications = new HashMap<String, String>();
    private HashMap<String, String> searchEngineCharge = new HashMap<String, String>();
    private HashMap<String, String> searchEngineValidation = new HashMap<String, String>();

    private HashMap<String, ArrayList<String>> tags = new HashMap<String, ArrayList<String>>();
    private HashMap<String, ArrayList<String>> peptides = new HashMap<String, ArrayList<String>>();
    private HashMap<String, ArrayList<String>> charges = new HashMap<String, ArrayList<String>>();

    public static void main(String[] args) {

        TagMapping m = new TagMapping();

        try {

            System.out.println(new Date() + "Loading search engine results");
            m.loadSETags();

            System.out.println(new Date() + "Loading tag results");
            m.loadTags();

            System.out.println(new Date() + "Mapping results");
            m.exportTags();

            System.out.println("Mapping completed");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void loadSETags() throws FileNotFoundException, IOException {

        File inFile = new File("D:\\projects\\denovoGUI\\hela\\shaken\\no d.txt");
        BufferedReader br = new BufferedReader(new FileReader(inFile));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                if (!line.equals("")) {
                    String[] split = line.split(separator);
                    String spectrumTitle = split[7];
                    String tag = split[19];
                    String peptide = split[3];
                    String modifications = split[5];
                    String charge = split[10];
                    String validation = split[22];
                    searchEngineTags.put(spectrumTitle, tag);
                    searchEnginePeptides.put(spectrumTitle, peptide);
                    searchEngineModifications.put(spectrumTitle, modifications);
                    searchEngineCharge.put(spectrumTitle, charge);
                    searchEngineValidation.put(spectrumTitle, validation);
                }
            }
        } finally {
            br.close();
        }
    }

    public void loadTags() throws IOException {

        File inFile = new File("D:\\projects\\denovoGUI\\hela\\shaken\\d assumptions.txt");
        BufferedReader br = new BufferedReader(new FileReader(inFile));
        try {
            String line = br.readLine();
            while ((line = br.readLine()) != null) {
                if (!line.equals("")) {
                    String[] split = line.split(separator);
                    String spectrumTitle = split[7];
                    String sequence = split[3];
                    String charge = split[10];
                    int startIndex = sequence.indexOf(">");
                    if (startIndex < sequence.length() && startIndex > -1) {
                        sequence = sequence.substring(startIndex + 1);
                    }
                    int endIndex = sequence.lastIndexOf("<");
                    if (endIndex > -1) {
                        sequence = sequence.substring(0, endIndex);
                    }
                    if (endIndex == -1 && startIndex == -1) {
                        ArrayList<String> spectrumPeptides = peptides.get(spectrumTitle);
                        if (spectrumPeptides == null) {
                            spectrumPeptides = new ArrayList<String>();
                            peptides.put(spectrumTitle, spectrumPeptides);
                        }
                        spectrumPeptides.add(sequence);
                    } else {
                        ArrayList<String> spectrumTags = tags.get(spectrumTitle);
                        if (spectrumTags == null) {
                            spectrumTags = new ArrayList<String>();
                            tags.put(spectrumTitle, spectrumTags);
                        }
                        spectrumTags.add(sequence);
                    }
                    ArrayList<String> spectrumCharges = charges.get(spectrumTitle);
                    if (spectrumCharges == null) {
                        spectrumCharges = new ArrayList<String>();
                        charges.put(spectrumTitle, spectrumCharges);
                    }
                    spectrumCharges.add(charge);
                }
            }
        } finally {
            br.close();
        }
    }

    public void exportTags() throws IOException {
        
        SequenceMatchingPreferences sequenceMatchingPreferences = new SequenceMatchingPreferences();
        sequenceMatchingPreferences.setSequenceMatchingType(SequenceMatchingPreferences.MatchingType.indistiguishableAminoAcids);
        sequenceMatchingPreferences.setMs2MzTolerance(0.1);

        File outFile = new File("D:\\projects\\denovoGUI\\hela\\shaken\\mapping.txt");
        BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
        try {

            bw.write("Spectrum" + separator + "Peptide Sequence" + separator + "Modifications" + separator + "Search engine charge" + separator + "Search Engine Tag" + separator + "length" + separator + "Validation" + separator + "DirecTag tag" + separator + "DirecTag peptide" + separator + "DirecTag charge" + separator + "Tag Found" + separator + "Peptide Found" + separator + "Charge Found" + separator + "Reversed");
            bw.newLine();

            for (String spectrum : searchEngineTags.keySet()) {
                String sePeptide = searchEnginePeptides.get(spectrum);
                String modifications = searchEngineModifications.get(spectrum);
                String seCharge = searchEngineCharge.get(spectrum);
                String seTag = searchEngineTags.get(spectrum);
                String seValidation = searchEngineValidation.get(spectrum);
                AminoAcidPattern aminoAcidPattern = new AminoAcidPattern(seTag);
                String tag = "";
                int sameTag = 0;
                String charge = "";
                int sameCharge = 0;
                int reversed = 0;
                ArrayList<String> direcTags = tags.get(spectrum);
                ArrayList<String> directCharges = charges.get(spectrum);
                if (direcTags != null) {
                    for (int i = 0; i < direcTags.size(); i++) {
                        String direcTag = direcTags.get(i);
                        String reversedTag = (new StringBuilder(direcTag)).reverse().toString();
                        String directCharge = directCharges.get(i);
                        if (seTag.contains(direcTag)) {
                            tag = direcTag;
                            sameTag = 1;
                            charge = directCharge;
                            if (seCharge.equals(directCharge)) {
                                sameCharge = 1;
                            } else {
                                sameCharge = 0;
                            }
                            reversed = 0;
                            break;
                        } else if (aminoAcidPattern.contains(direcTag, sequenceMatchingPreferences)) {
                            tag = direcTag;
                            sameTag = 1;
                            charge = directCharge;
                            if (seCharge.equals(directCharge)) {
                                sameCharge = 1;
                            } else {
                                sameCharge = 0;
                            }
                            reversed = 0;
                        }else if (aminoAcidPattern.contains(reversedTag, sequenceMatchingPreferences)) {
                            tag = direcTag;
                            sameTag = 1;
                            charge = directCharge;
                            if (seCharge.equals(directCharge)) {
                                sameCharge = 1;
                            } else {
                                sameCharge = 0;
                            }
                            reversed = 1;
                        } else if (tag.equals("")) {
                            tag = direcTag;
                        }
                        if (sameCharge == 0) {
                            charge = directCharge;
                            if (seCharge.equals(directCharge)) {
                                sameCharge = 1;
                            } else {
                                sameCharge = 0;
                            }
                        }
                    }
                }
                String peptide = "";
                int samePeptide = 0;
                ArrayList<String> directPeptides = peptides.get(spectrum);
                aminoAcidPattern = new AminoAcidPattern(sePeptide);
                if (directPeptides != null) {
                    for (String direcTag : directPeptides) {
                        if (aminoAcidPattern.contains(direcTag, sequenceMatchingPreferences)) {
                            peptide = direcTag;
                            samePeptide = 1;
                            break;
                        } else if (peptide.equals("")) {
                            peptide = direcTag;
                        }
                    }
                }

                bw.write(spectrum + separator + sePeptide + separator + modifications + separator + seCharge + separator + seTag + separator + seTag.length() + separator + seValidation + separator + tag + separator + peptide + separator + charge + separator + sameTag + separator + samePeptide + separator + sameCharge + separator + reversed);
                bw.newLine();

            }

        } finally {
            bw.close();
        }
    }

}
